/* Poly-Raster Image Format definitions & macros
 *
 * Copyright (c) 2006-2010 ITB CompuPhase, The Netherlands, www.compuphase.com
 *
 */
#ifndef PRIF_H
#define PRIF_H

#if defined _WIN32 || defined __WIN32 || defined __WIN32__
    #pragma pack(push)
    #pragma pack(1)     /* structures must be packed (byte-aligned) */
#endif
#if defined __TURBOC__
  #pragma option -a-    /* "pack" pragma for older Borland compilers */
#endif

/* C/C++ structures
 * If you use these for file I/O, make sure that:
 *   1. the compiler packs the structure
 *   2. the target CPU is Little Endian architecture
 * It is advised to use the macros below for best portability. See the
 * examples in the poly-raster archive for examples.
 */
typedef struct tagPRHDR {
    unsigned long  size;          /* bitmap size in the file */
    unsigned short id;            /* must be 0xA202 */
    unsigned char  layout;        /* file lay-out and format flags */
    unsigned char  depth;         /* colour depth, bits per pixels */
    unsigned short width;
    unsigned short height;
} PRHDR;

typedef struct tagPREXTHDR {
    unsigned short delay;         /* milliseconds to wait before displaying this bitmap */
    unsigned short dx;            /* value to add to the x-position of the image */
    unsigned short dy;            /* value to add to the y-position of the image */
} PREXTHDR;

#if defined _WIN32 || defined __WIN32 || defined __WIN32__
    #pragma pack(pop)
#endif


/* macros to extract fields from the header
 * these handle Big-Endian versus Little-Endian issues too
 */
#define PRHDR_FIELD8(b,f)   (((unsigned char*)(b))[(f)])

#define PRHDR_FIELD16(b,f)  ( (unsigned short) (((unsigned char*)(b))[(f)])         \
                            | (unsigned short)((((unsigned char*)(b))[(f)+1]) << 8) )

#define PRHDR_FIELD32(b,f)  ( (unsigned long) (((unsigned char*)(b))[(f)])          \
                            | (unsigned long)((((unsigned char*)(b))[(f)+1]) << 8)  \
                            | (unsigned long)((((unsigned char*)(b))[(f)+2]) << 16) \
                            | (unsigned long)((((unsigned char*)(b))[(f)+3]) << 24) )

#define PRHDR_SIZE(b)       PRHDR_FIELD32(b,0x00)
#define PRHDR_ID(b)         PRHDR_FIELD16(b,0x04)
#define PRHDR_LAYOUT(b)     PRHDR_FIELD8(b,0x06)
#define PRHDR_DEPTH(b)      PRHDR_FIELD8(b,0x07)
#define PRHDR_WIDTH(b)      PRHDR_FIELD16(b,0x08)
#define PRHDR_HEIGHT(b)     PRHDR_FIELD16(b,0x0a)

#define PREXTHDR_DELAY(b)   PRHDR_FIELD16(b,0x00)
#define PREXTHDR_DX(b)      PRHDR_FIELD16(b,0x02)
#define PREXTHDR_DY(b)      PRHDR_FIELD16(b,0x04)

#define PRHEADERSIZE        12
#define PREXTHEADERSIZE     6
#define PRSIGNATURE         0xa202

#define PRLAYOUT_COLUMN     0x01
#define PRLAYOUT_BANDED     0x02
#define PRLAYOUT_REVERSE    0x04
#define PRLAYOUT_PLANAR     0x08
#define PRLAYOUT_YINVERT    0x10
#define PRLAYOUT_EXTHDR     0x20
#define PRLAYOUT_CMAP       0x40
#define PRLAYOUT_LOOPBACK   0x80

#define PRBITMAPSIZE(prhdr) ( ((prhdr).depth < 8) ? \
                                 (((((prhdr).width + 8 / (prhdr).depth - 1) * (prhdr).depth) / 8) * (prhdr).height) : \
                                 ((prhdr).width * (prhdr).height) \
                            )

#endif /* PRIF_H */
